/* client.c -- simple example of a client application using socket(2)
   interface over TCP/IP and INET */

/* Brad Karp and Brad Chen */

#define MYSOCK		80	/* HTTPD */
#define BUFSIZE		4096

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>

extern struct hostent *gethostbyname();

#define ASSERT(c, s) \
  if (!(c)) {	perror(s);	exit(1);	}

void
main(int argc, char *argv[])
{
  int sockdes;			/* descriptor for our socket */
  struct sockaddr_in client;	/* the configuration params for our socket */
  struct hostent *he;		/* host table entry for our destination */
  char buf[BUFSIZE];
  int rval;

  /* grab a socket that can go between machines and is a stream, using TCP */
  sockdes = socket(AF_INET, SOCK_STREAM, 0);
  ASSERT(sockdes >= 0, "open socket");

  /* configure socket */
  client.sin_family = AF_INET;
  he = gethostbyname(argv[1]);
  ASSERT(he != 0, "Unknown host");

  bcopy((char *) he->h_addr, (char *) &client.sin_addr, he->h_length);
  client.sin_port = htons(MYSOCK);

  /* Establish connection */
  rval = connect(sockdes, (struct sockaddr *) &client, sizeof(client));
  ASSERT(rval >= 0, "connect to socket");

  /* Send */
  if (fgets(buf, BUFSIZE, stdin) == NULL) exit(0);

  fprintf(stderr, "writing [%s]\n", buf);
  
  rval = write(sockdes, buf, strlen(buf));
  ASSERT(rval == strlen(buf), "write to socket");

  rval = write(sockdes, "\r\n\r\n", 4);
  ASSERT(rval == 4, "write to socket");

  /* Receive */
  while (1) {
    rval = read(sockdes, buf, BUFSIZE);
    ASSERT(rval >= 0, "read from socket");

    if (rval == 0) break;
    fwrite(buf, rval, 1, stdout);
  }

  close(sockdes);
  exit(0);
}
